//
// Created by anderlim on 07.02.17.
//

#include <omp.h>
#include <cgp.h>
#include <stdio.h>
#include <sys/stat.h>
#include "../src/cgp.h"
#include "../src/sm-functions.h"
#include "../src/fitnessFunctions.h"

int main(int argc, char* argv[]){


    char* path = argv[1];

    if(path != NULL){
        mkdir(path, S_IRWXU | S_IRWXG | S_IRWXO);
    }

    struct parameters *params = NULL;
    struct dataSet *trainingData = NULL;
    struct chromosome *chromoA = NULL;
    struct chromosome *chromoB = NULL;
    struct chromosome *chromoC = NULL;

    int numInputs = 1;
    int numNodes = 20;
    int numOutputs = 1;
    int nodeArity = 2;

    int numGens = 1000;
    double targetFitness = 0.0;
    int updateFrequency = 10;

//setenv("PYTHONPATH","src",1);

    parityInit();

    params = initialiseParameters(numInputs, numNodes, numOutputs, nodeArity);

    params->numSmActivations = 1;

    setCustomFitnessFunction(params, computationalFitness, "CompFitness");

    omp_set_nested(1);
    setNumThreads(params, 8);

    setMu(params, 2);
    setLambda(params, 8);

    addNodeFunction(params, "add, sub, mul, sin, cos, sm-movr, sm-dupr, sm-mov, sm-dup, sm-del, index");

    setNumDevRuns(params, 50);

    setNumAverage(params, 1);

    setTargetFitness(params, targetFitness);

    setMutationRate(params, 0.05);

    setMutationType(params, "point");

    setUpdateFrequency(params, updateFrequency);

    setRecurrentConnectionProbability(params, 0.4);

    setShortcutConnections(params, 0);

    printParameters(params);

    // Note: you may need to check this path such that it is relative to your executable
    trainingData = initialiseDataSetFromFile("./dataSets/tempParity.data");

    chromoA = runCGP(params, trainingData, numGens);

    //chromoA = initialiseChromosome(params);
    //printChromosome(chromoA, 0);

   const int inputs[1] = {0.0};

//    chromoB = initialiseChromosomeFromChromosome(chromoA);
//    for(int i = 0; i < 20; i++){
//        //printf("Iteration %d\n", i);
//        chromoB = executeChromosome(chromoB, getDataSetSampleInputs(trainingData, i));
//    }
//    chromoC = initialiseChromosomeFromChromosome(chromoA);
//    for(int i = 0; i < 20; i++){
//        //printf("Iteration %d\n", i);
//        chromoC = executeChromosome(chromoC, getDataSetSampleInputs(trainingData, i));
//    }

    //printChromosome(chromoB, 0);
    //printChromosome(chromoC, 0);
    chromoC = initialiseChromosomeFromChromosome(chromoA);

    printf("Chromo length: ");
    for(int j = 0; j<10; j++){
        for(int i = 0; i < 20; i++){
            //printf("Iteration %d\n", i);
            chromoC = executeChromosome(chromoC, getDataSetSampleInputs(trainingData, i));
        }
        printf("%d, ", chromoC->numNodes);
        chromoC = initialiseChromosomeFromChromosome(chromoA);
    }
    printf("\nChromo length: ");
    for(int j = 0; j<10; j++){
        for(int i = 0; i < 50; i++){
            //printf("Iteration %d\n", i);
            chromoC = executeChromosome(chromoC, getDataSetSampleInputs(trainingData, i));
        }
        printf("%d, ", chromoC->numNodes);
        chromoC = initialiseChromosomeFromChromosome(chromoA);
    }
    printf("\nChromo length: ");
    for(int j = 0; j<10; j++){
        for(int i = 0; i <200; i++){
            //printf("Iteration %d\n", i);
            chromoC = executeChromosome(chromoC, getDataSetSampleInputs(trainingData, i));
        }
        printf("%d, ", chromoC->numNodes);
        chromoC = initialiseChromosomeFromChromosome(chromoA);
    }

//    chromoC = _MOV(chromoA, 0, &inputs, params);
//    printChromosome(chromoC, 0);


    char chrFilename[] = "bestChromo.chr";
    char dotFilename[] = "bestChromo.dot";
    char parFilename[] = "bestChromo.par";

    char chrPath[50];
    char dotPath[50];
    char parPath[50];

    sprintf(chrPath, "%s/%s", path, chrFilename);
    sprintf(dotPath, "%s/%s", path, dotFilename);
    sprintf(parPath, "%s/%s", path, parFilename);

    saveChromosome(chromoA, chrPath);
    saveChromosomeDot(chromoA, 0, dotPath);
    saveParameters(params, parPath);
    printf("\n\nWrote files: %s,%s\n",chrPath, dotPath);
    freeDataSet(trainingData);
    freeChromosome(chromoA);
    freeChromosome(chromoB);
    freeChromosome(chromoC);
    freeParameters(params);

    parityFinalize();

    return 0;
}
