//
// Moved from cgp.c
//

#include "utils.h"

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

/*
	Node function add. Returns the sum of all the inputs.
*/
double _add(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    double sum = inputs[0];

    for (i = 1; i < numInputs; i++) {
        sum += inputs[i];
    }

    return sum;
}

/*
	Node function sub. Returns the first input minus all remaining inputs.
*/
double _sub(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    double sum = inputs[0];

    for (i = 1; i < numInputs; i++) {
        sum -= inputs[i];
    }

    return sum;
}


/*
	Node function mul. Returns the multiplication of all the inputs.
*/
double _mul(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    double multiplication = inputs[0];

    for (i = 1; i < numInputs; i++) {
        multiplication *= inputs[i];
    }

    return multiplication;
}


/*
	Node function div. Returns the first input divided by the second input divided by the third input etc
*/
double _divide(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    double divide = inputs[0];

    for (i = 1; i < numInputs; i++) {
        divide /= inputs[i];
    }

    return divide;
}


/*
	Node function abs. Returns the absolute of the first input
*/
double _absolute(const int numInputs, const double *inputs, const double *connectionWeights) {

    return fabs(inputs[0]);
}


/*
	Node function sqrt.  Returns the square root of the first input
*/
double _squareRoot(const int numInputs, const double *inputs, const double *connectionWeights) {

    return sqrt(inputs[0]);
}


/*
	Node function squ.  Returns the square of the first input
*/
double _square(const int numInputs, const double *inputs, const double *connectionWeights) {

    return pow(inputs[0], 2);
}


/*
	Node function cub.  Returns the cube of the first input
*/
double _cube(const int numInputs, const double *inputs, const double *connectionWeights) {

    return pow(inputs[0], 3);
}


/*
	Node function power.  Returns the first output to the power of the second
*/
double _power(const int numInputs, const double *inputs, const double *connectionWeights) {

    return pow(inputs[0], inputs[1]);
}

/*
	Node function exp.  Returns the exponential of the first input
*/
double _exponential(const int numInputs, const double *inputs, const double *connectionWeights) {

    return exp(inputs[0]);
}


/*
	Node function sin.  Returns the sine of the first input
*/
double _sine(const int numInputs, const double *inputs, const double *connectionWeights) {

    return sin(inputs[0]);
}

/*
	Node function cos.  Returns the cosine of the first input
*/
double _cosine(const int numInputs, const double *inputs, const double *connectionWeights) {

    return cos(inputs[0]);
}

/*
	Node function tan.  Returns the tangent of the first input
*/
double _tangent(const int numInputs, const double *inputs, const double *connectionWeights) {

    return tan(inputs[0]);
}


/*
	Node function one.  Always returns 1
*/
double _constOne(const int numInputs, const double *inputs, const double *connectionWeights) {
    return 1;
}

/*
	Node function one.  Always returns 0
*/
double _constZero(const int numInputs, const double *inputs, const double *connectionWeights) {
    return 0;
}

/*
	Node function one.  Always returns PI
*/
double _constPI(const int numInputs, const double *inputs, const double *connectionWeights) {
    return M_PI;
}


/*
	Node function rand.  Returns a random number between minus one and positive one
*/
double _randFloat(const int numInputs, const double *inputs, const double *connectionWeights) {

    double random;

    random = (double)rand() / (double)RAND_MAX;

    random = (random * 2) - 1;

    return random;
}

/*
	Node function and. logical AND, returns '1' if all inputs are '1'
	else, '0'
*/
double _and(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;

    for (i = 0; i < numInputs; i++) {

        if (inputs[i] == 0) {
            return 0;
        }
    }

    return 1;
}

/*
	Node function and. logical NAND, returns '0' if all inputs are '1'
	else, '1'
*/
double _nand(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;

    for (i = 0; i < numInputs; i++) {

        if (inputs[i] == 0) {
            return 1;
        }
    }

    return 0;
}


/*
	Node function or. logical OR, returns '0' if all inputs are '0'
	else, '1'
*/
double _or(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;

    for (i = 0; i < numInputs; i++) {

        if (inputs[i] == 1) {
            return 1;
        }
    }

    return 0;
}


/*
	Node function nor. logical NOR, returns '1' if all inputs are '0'
	else, '0'
*/
double _nor(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;

    for (i = 0; i < numInputs; i++) {

        if (inputs[i] == 1) {
            return 0;
        }
    }

    return 1;
}


/*
	Node function xor. logical XOR, returns '1' iff one of the inputs is '1'
	else, '0'. AKA 'one hot'.
*/
double _xor(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    int numOnes = 0;
    int out;

    for (i = 0; i < numInputs; i++) {

        if (inputs[i] == 1) {
            numOnes++;
        }

        if (numOnes > 1) {
            break;
        }
    }

    if (numOnes == 1) {
        out = 1;
    }
    else {
        out = 0;
    }

    return out;
}

/*
	Node function xnor. logical XNOR, returns '0' iff one of the inputs is '1'
	else, '1'.
*/
double _xnor(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    int numOnes = 0;
    int out;

    for (i = 0; i < numInputs; i++) {

        if (inputs[i] == 1) {
            numOnes++;
        }

        if (numOnes > 1) {
            break;
        }
    }

    if (numOnes == 1) {
        out = 0;
    }
    else {
        out = 1;
    }

    return out;
}

/*
	Node function not. logical NOT, returns '1' if first input is '0', else '1'
*/
double _not(const int numInputs, const double *inputs, const double *connectionWeights) {

    double out;

    if (inputs[0] == 0) {
        out = 1;
    }
    else {
        out = 0;
    }

    return out;
}


/*
	Node function wire. simply acts as a wire returning the first input
*/
double _wire(const int numInputs, const double *inputs, const double *connectionWeights) {

    double out;

    out = inputs[0];

    return out;
}


/*
	Node function sigmoid. returns the sigmoid of the sum of weighted inputs.
	The specific sigmoid function used in the logistic function.
	range: [0,1]
*/
double _sigmoid(const int numInputs, const double *inputs, const double *connectionWeights) {

    double weightedInputSum;
    double out;

    weightedInputSum = sumWeightedInputs(numInputs, inputs, connectionWeights);

    out = 1 / (1 + exp(-weightedInputSum));

    return out;
}

/*
	Node function Gaussian. returns the Gaussian of the sum of weighted inputs.
	range: [0,1]
*/
double _gaussian(const int numInputs, const double *inputs, const double *connectionWeights) {

    double weightedInputSum;
    double out;

    int centre = 0;
    int width = 1;

    weightedInputSum = sumWeightedInputs(numInputs, inputs, connectionWeights);

    out = exp(-(pow(weightedInputSum - centre, 2)) / (2 * pow(width, 2)));

    return out;
}


/*
	Node function step. returns the step function of the sum of weighted inputs.
	range: [0,1]
*/
double _step(const int numInputs, const double *inputs, const double *connectionWeights) {

    double weightedInputSum;
    double out;

    weightedInputSum = sumWeightedInputs(numInputs, inputs, connectionWeights);

    if (weightedInputSum < 0) {
        out = 0;
    }
    else {
        out = 1;
    }

    return out;
}


/*
	Node function step. returns the step function of the sum of weighted inputs.
	range: [-1,1]
*/
double _softsign(const int numInputs, const double *inputs, const double *connectionWeights) {

    double weightedInputSum;
    double out;

    weightedInputSum = sumWeightedInputs(numInputs, inputs, connectionWeights);

    out = weightedInputSum / (1 + fabs(weightedInputSum));

    return out;
}


/*
	Node function tanh. returns the tanh function of the sum of weighted inputs.
	range: [-1,1]
*/
double _hyperbolicTangent(const int numInputs, const double *inputs, const double *connectionWeights) {

    double weightedInputSum;
    double out;

    weightedInputSum = sumWeightedInputs(numInputs, inputs, connectionWeights);

    out = tanh(weightedInputSum);

    return out;
}
