//
// Created by anderlim on 07.02.17.
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sm-functions.h"
#include "cgp.h"

#define MAX_SIZE 400

void copyOutputs(struct chromosome* dest, struct chromosome* src){
    for (int i = 0; i < src->numNodes; i++){
        dest->nodes[i]->output = src->nodes[i]->output;
    }
}

double sm_comp(const int numInputs, const double *inputs, const double *connectionWeights) {
    if(inputs[0]<inputs[1]){
        return 0.0;
    }
    return 1.0;
}

struct chromosome *_ADD(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS]){

    int i;

    int num_nodes = getNumChromosomeNodes(chromo_in);

    int numNewNodes = sm_args[1];
    int newNodesOffset = sm_args[0];


//    Set minimum offset to first non input node
    if(newNodesOffset + index < 0)
        newNodesOffset = -index-1;

//    Set maximum offset to the last node
    if(newNodesOffset + index >= chromo_in->numNodes)
        newNodesOffset = chromo_in->numNodes - index - 1;


    struct chromosome *chromo_out = initialiseChromosomeFromChromosome(chromo_in);
    copyOutputs(chromo_out, chromo_in);
    if(numNewNodes<1 || num_nodes > MAX_SIZE) return chromo_out;

    //increase node count
    chromo_out->numNodes+=numNewNodes;

    // realloc nodes and smNodes memory
    reallocNodes(chromo_out, num_nodes+numNewNodes);

    // move nodes after insertion point to the end of list
    for (i = 1; i <= num_nodes-(index+newNodesOffset); i++){
        chromo_out->nodes[num_nodes+numNewNodes-i] = chromo_out->nodes[num_nodes-i];
        chromo_out->smNodes[num_nodes+numNewNodes-i] = chromo_out->smNodes[num_nodes-i];
    }

    //create and insert new node(s) into new chromo OK
    for (i = index+newNodesOffset+1; i <=index+newNodesOffset+numNewNodes; i++){
        //create new node(s)
        struct node *newNode = initialiseNode(chromo_out->numInputs,chromo_out->numNodes,chromo_out->arity,chromo_out->funcSet->numFunctions, 0.0, chromo_out->recurrentConnectionProbability,i, chromo_out->smNodes);
        //insert node
        chromo_out->nodes[i] = newNode;
    }


    checkChromosomeInputs(chromo_out);

    setChromosomeActiveNodes(chromo_out);

    return chromo_out;

}

struct chromosome *_DEL(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS]){

    int i;

    int delStart = index + sm_args[0] + 1;
    int delEnd = index + sm_args[0] + sm_args[1];

    // Check boundaries
    if(delStart < 0) delStart = 0;
    if(delEnd > chromo_in->numNodes) delEnd = chromo_in->numNodes-1;

    int numDel = delEnd - delStart;
    //Return if not deleting any node

    int num_nodes = getNumChromosomeNodes(chromo_in);
    //printf("num_nodes: %d\n", num_nodes);

    struct chromosome *chromo_out = initialiseChromosomeFromChromosome(chromo_in);
    copyOutputs(chromo_out, chromo_in);
    if (numDel<1) return chromo_out;

    //reduce node count
    chromo_out->numNodes-=numDel;

    //delete node
    for(i = delStart; i < delEnd; i++) {
        freeNode(chromo_out->nodes[i]);
    }

    //remove node from chromo
    for(i = delStart; i < chromo_out->numNodes; i++){
        chromo_out->nodes[i] = chromo_out->nodes[i+numDel];
        chromo_out->smNodes[i] = chromo_out->smNodes[i+numDel];
    }

    // realloc structures
    struct node** tmp = realloc(chromo_out->nodes, sizeof(struct node*) * (num_nodes-numDel)+1);
    if(!tmp){
        fprintf(stderr, "Realloc failed!\n");
    }
    chromo_out->nodes = tmp;

    int *tmp2 = realloc(chromo_out->smNodes, sizeof(int) * (num_nodes-numDel)+1);
    if(!tmp2){
        fprintf(stderr, "Realloc failed!\n");
    }
    chromo_out->smNodes = tmp2;

    int *tmp3 = realloc(chromo_out->activeNodes, sizeof(int) * (num_nodes-numDel)+1);
    if(!tmp3){
        fprintf(stderr, "Realloc failed!\n");
    }
    chromo_out->activeNodes = tmp3;


    num_nodes = chromo_out->numNodes;
    /*
     * Fix inconsistencies in graph (dont point to deleted nodes
     * */
    // Check outputs
    for(i = 0; i < chromo_out->numOutputs; i++){
        if(chromo_out->outputNodes[i] > num_nodes+chromo_out->numInputs-1){
            chromo_out->outputNodes[i] = num_nodes+chromo_out->numInputs-1;
            //printf("\x1b[33mFIX: Output node out out of bounds fixed, now: %d\x1b[0m\n", chromo_out->outputNodes[i]);
        }
    }

    checkChromosomeInputs(chromo_out);


    setChromosomeActiveNodes(chromo_out);

    return chromo_out;

}

struct chromosome *_MOV(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS]){

    int i,j;
    int num_nodes = getNumChromosomeNodes(chromo_in);

    int movStart = index + sm_args[0];
    int movEnd = index + sm_args[0] + sm_args[1];
    int insert = index + sm_args[0] + sm_args[2];

    // Check boundaries
    if(movStart < 0) movStart = 0;
    if(movEnd > chromo_in->numNodes) movEnd = chromo_in->numNodes-1;

    int numMov = movEnd - movStart;

    if(insert+numMov>num_nodes) insert = num_nodes-numMov-1;
    if(insert<0) insert = 0;
    //Return if not deleting any node

    //printf("num_nodes: %d\n", num_nodes);

    struct chromosome *chromo_out = initialiseChromosomeFromChromosome(chromo_in);
    copyOutputs(chromo_out, chromo_in);
    if (numMov<1 || (insert >= movStart && insert < movEnd)) return chromo_out;

    struct node** nodesCopy = malloc(num_nodes * sizeof(struct node*));
    memcpy(nodesCopy, chromo_in->nodes, num_nodes * sizeof(struct node*));

    int* smNodesCopy = malloc(num_nodes * sizeof(int));
    memcpy(smNodesCopy, chromo_in->smNodes, num_nodes * sizeof(int));

    j = 0;

    if(insert < movStart) {
        for (i = 0; i < insert; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movStart; i < movEnd; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = insert; i < movStart ; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movEnd; i < chromo_out->numNodes; i++){
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }


    }else{
        for(i = 0; i< movStart; i++){
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movEnd; i < insert; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movStart; i < movEnd; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = insert; i < chromo_out->numNodes; i++){
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }
    }


    free(nodesCopy);
    free(smNodesCopy);

    checkChromosomeInputs(chromo_out);

    setChromosomeActiveNodes(chromo_out);

    return chromo_out;
}

struct chromosome *_MOVR(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS]){

    int i,j;
    int num_nodes = getNumChromosomeNodes(chromo_in);

    int movStart = index + sm_args[0];
    int movEnd = index + sm_args[0] + sm_args[1];
    int insert = index + sm_args[0] + sm_args[2];

    // Check boundaries
    if(movStart < 0) movStart = 0;
    if(movEnd > chromo_in->numNodes) movEnd = chromo_in->numNodes-1;

    int numMov = movEnd - movStart;

    if(insert+numMov>num_nodes) insert = num_nodes-numMov-1;
    if(insert<0) insert = 0;

    int relativeInp = 0;
    //Return if not deleting any node

    //printf("num_nodes: %d\n", num_nodes);

    struct chromosome *chromo_out = initialiseChromosomeFromChromosome(chromo_in);
    copyOutputs(chromo_out, chromo_in);
    if (numMov<1 || (insert >= movStart && insert < movEnd)) return chromo_out;

    struct node** nodesCopy = malloc(num_nodes * sizeof(struct node*));
    memcpy(nodesCopy, chromo_in->nodes, num_nodes * sizeof(struct node*));

    int* smNodesCopy = malloc(num_nodes * sizeof(int));
    memcpy(smNodesCopy, chromo_in->smNodes, num_nodes * sizeof(int));


    //update relative inputs
    for(i = 0; i<numMov; i++){
        for(j = 0; j<chromo_in->nodes[movStart+i]->actArity; j++){
            relativeInp = chromo_in->nodes[movStart+i]->inputs[j] - (movStart+i);
            nodesCopy[i]->inputs[j] = insert + i + relativeInp;
        }
    }

    j = 0;

    if(insert < movStart) {
        for (i = 0; i < insert; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movStart; i < movEnd; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = insert; i < movStart ; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movEnd; i < chromo_out->numNodes; i++){
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }


    }else{
        for(i = 0; i< movStart; i++){
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movEnd; i < insert; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = movStart; i < movEnd; i++) {
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }

        for (i = insert; i < chromo_out->numNodes; i++){
            copyNode(chromo_out->nodes[j], nodesCopy[i]);
            chromo_out->smNodes[j] = smNodesCopy[i];
            j++;
        }
    }



    free(nodesCopy);
    free(smNodesCopy);

    checkChromosomeInputs(chromo_out);

    setChromosomeActiveNodes(chromo_out);

    return chromo_out;
}

struct chromosome *_DUP(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS]){

    int i;
    int num_nodes = getNumChromosomeNodes(chromo_in);

    int dupStart = index + sm_args[0];
    int dupEnd = index + sm_args[0] + sm_args[1];
    int insert = index + sm_args[0] + sm_args[2];

    // Check boundaries
    if(dupStart < 0) dupStart = 0;
    if(dupEnd > num_nodes) dupEnd = num_nodes-1;

    int numDup = dupEnd - dupStart;

    if(insert >= num_nodes) insert = num_nodes - 1;
    if(insert < 0) insert = 0;
    //Return if not deleting any node

    //printf("num_nodes: %d\n", num_nodes);

    struct chromosome *chromo_out = initialiseChromosomeFromChromosome(chromo_in);
    copyOutputs(chromo_out, chromo_in);
    if (dupEnd <= dupStart  || num_nodes > MAX_SIZE) return chromo_out;

    struct node** nodesCopy = malloc(numDup * sizeof(struct node*));
    int* smNodesCopy = malloc(numDup * sizeof(int));

    for(i = 0; i < numDup; i++){
        nodesCopy[i] = initialiseNode(chromo_out->numInputs, num_nodes,chromo_out->arity,chromo_out->funcSet->numFunctions, 1.0, chromo_out->recurrentConnectionProbability, i+insert,chromo_out->smNodes);
        copyNode(nodesCopy[i], chromo_in->nodes[dupStart+i]);
        smNodesCopy[i] = chromo_in->smNodes[dupStart+i];
    }

    // realloc nodes and smNodes memory
    reallocNodes(chromo_out, num_nodes+numDup);


    // move nodes after insertion point to the end of list
    for (i = num_nodes-1; i > insert; i--){
        chromo_out->nodes[i+numDup] = chromo_out->nodes[i];
        chromo_out->smNodes[i+numDup] = chromo_out->smNodes[i];
    }

    // insert duplicated nodes
    for (i = 0; i < numDup; i++){
        chromo_out->nodes[insert+i+1] = nodesCopy[i];
        chromo_out->smNodes[insert+i+1] = smNodesCopy[i];
    }

    chromo_out->numNodes = num_nodes+numDup;

    free(nodesCopy);
    free(smNodesCopy);

    checkChromosomeInputs(chromo_out);

    setChromosomeActiveNodes(chromo_out);

    return chromo_out;
}

struct chromosome *_DUPR(struct chromosome *chromo_in, int index, int sm_args[NUM_ARGUMENTS]){

    int i;
    int num_nodes = getNumChromosomeNodes(chromo_in);

    int dupStart = index + sm_args[0];
    int dupEnd = index + sm_args[0] + sm_args[1];
    int insert = index + sm_args[0] + sm_args[2];

    // Check boundaries
    if(dupStart < 0) dupStart = 0;
    if(dupEnd > num_nodes) dupEnd = num_nodes-1;

    int numDup = dupEnd - dupStart;

    if(insert >= num_nodes) insert = num_nodes - 1;
    if(insert < 0) insert = 0;

    int relativeInp = 0;
    //Return if not deleting any node

    //printf("num_nodes: %d\n", num_nodes);

    struct chromosome *chromo_out = initialiseChromosomeFromChromosome(chromo_in);
    copyOutputs(chromo_out, chromo_in);
    if (dupEnd <= dupStart  || num_nodes > MAX_SIZE) return chromo_out;

    struct node** nodesCopy = malloc(numDup * sizeof(struct node*));
    int* smNodesCopy = malloc(numDup * sizeof(int));

    for(i = 0; i < numDup; i++){
        nodesCopy[i] = initialiseNode(chromo_out->numInputs,num_nodes,chromo_out->arity,chromo_out->funcSet->numFunctions, 1.0, chromo_out->recurrentConnectionProbability, i+insert,chromo_out->smNodes);
        copyNode(nodesCopy[i], chromo_in->nodes[dupStart+i]);
        smNodesCopy[i] = chromo_in->smNodes[dupStart+i];

        //update relative inputs
        for(int j = 0; j<chromo_in->nodes[dupStart+i]->actArity; j++){
            relativeInp = chromo_in->nodes[dupStart+i]->inputs[j] - (dupStart+i);
            nodesCopy[i]->inputs[j] = insert + i + relativeInp;
        }
    }



    // realloc nodes and smNodes memory
    reallocNodes(chromo_out, num_nodes+numDup);


    // move nodes after insertion point to the end of list
    for (i = num_nodes-1; i > insert; i--){
        chromo_out->nodes[i+numDup] = chromo_out->nodes[i];
        chromo_out->smNodes[i+numDup] = chromo_out->smNodes[i];
    }

    // insert duplicated nodes
    for (i = 0; i < numDup; i++){
        chromo_out->nodes[insert+i+1] = nodesCopy[i];
        chromo_out->smNodes[insert+i+1] = smNodesCopy[i];
    }

    chromo_out->numNodes = num_nodes+numDup;

    free(nodesCopy);
    free(smNodesCopy);

    checkChromosomeInputs(chromo_out);

    setChromosomeActiveNodes(chromo_out);

    return chromo_out;
}

double _SIZE (struct chromosome* chromo, int index, double info_weight){
    return (double)chromo->numNodes*info_weight;
}

double _INDEX (struct chromosome* chromo, int index, double info_weight){
    return (double)index*info_weight;
}

double _INPUTS(struct chromosome* chromo, int index, double info_weight){
    return (double)chromo->numInputs*info_weight;
}

// realloc nodes and smNodes memory
int reallocNodes(struct chromosome *chromo, int newSize) {
    struct node **tmp = realloc(chromo->nodes, sizeof(struct node *) * newSize);
    if (!tmp) {
        printf("Realloc nodes failed!\n");
    }
    chromo->nodes = tmp;

    int *tmp2 = realloc(chromo->smNodes, sizeof(int) * newSize);
    if (!tmp2) {
        printf("Realloc smNodes failed!\n");
    }
    chromo->smNodes = tmp2;

    int *tmp3 = realloc(chromo->activeNodes, sizeof(int) * newSize);
    if (!tmp3) {
        printf("Realloc activeNodes failed!\n");
    }
    chromo->activeNodes = tmp3;

    return 0;
}