//
// Moved from cgp.c
//

#include "utils.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>


/*
	returns a random decimal between [0,1]
*/
double randDecimal(void) {
    return (double)rand() / (double)RAND_MAX;
}

/*
	sort int array using qsort
*/
void sortIntArray(int *array, const int length) {

    qsort(array, length, sizeof(int), cmpInt);
}

/*
	used by qsort in sortIntArray
*/
int cmpInt(const void * a, const void * b) {
    return ( *(int*)a - * (int*)b );
}


/*
	sort double array using qsort
*/
void sortDoubleArray(double *array, const int length) {

    qsort(array, length, sizeof(double), cmpDouble);
}

/*
	used by qsort in sortDoubleArray
*/
int cmpDouble(const void * a, const void * b) {

    if ( *(double*)a < * (double*)b) {
        return -1;
    }
    if ( *(double*)a == *(double*)b ) {
        return 0;
    }
    else {
        return 1;
    }
}




/*
	random integer between zero and n without modulo bias.
	adapted from: http://zuttobenkyou.wordpress.com/2012/10/18/generating-random-numbers-without-modulo-bias/
*/
int randInt(int n) {

    int x;
    int randLimit;
    int randExcess;

    if (n == 0) {
        return 0;
    }

    randExcess = (RAND_MAX % n) + 1;
    randLimit = RAND_MAX - randExcess;

    do {
        x = rand();
    }
    while (x > randLimit);

    return x % n;
}

/*
	Returns the sum of the weighted inputs.
*/
double sumWeightedInputs(const int numInputs, const double *inputs, const double *connectionWeights) {

    int i;
    double weightedSum = 0;

    for (i = 0; i < numInputs; i++) {
        weightedSum += (inputs[i] * connectionWeights[i]);
    }

    return weightedSum;
}

double medianInt(const int *anArray, const int length) {

    int i;
    int *copyArray = (int*)malloc(length * sizeof(int));
    double median;

    /* make a copy of the array */
    for (i = 0; i < length; i++) {
        copyArray[i] = anArray[i];
    }

    /* sort the copy array */
    sortIntArray(copyArray, length);

    /* if even */
    if (length % 2 == 0) {
        median = (copyArray[(length / 2)] + copyArray[(length / 2) - 1] ) / 2;
    }

        /* if odd */
    else {
        median = copyArray[ (length - 1) / 2];
    }

    free(copyArray);

    return median;
}

double medianDouble(const double *anArray, const int length) {

    int i;
    double *copyArray = (double*)malloc(length * sizeof(double));
    double median;

    /* make a copy of the array */
    for (i = 0; i < length; i++) {
        copyArray[i] = anArray[i];
    }

    /* sort the copy array */
    sortDoubleArray(copyArray, length);

    /* if even */
    if (length % 2 == 0) {
        median = (copyArray[(length / 2)] + copyArray[(length / 2) - 1] ) / 2;
    }

        /* if odd */
    else {
        median = copyArray[ (length - 1) / 2];
    }

    free(copyArray);

    return median;
}


